Class: Flipper::Adapters::ActiveRecord

Inherits:
Object
  • Object
show all
Includes:
Flipper::Adapter
Defined in:
lib/flipper/adapters/active_record.rb,
lib/flipper/adapters/active_record/gate.rb,
lib/flipper/adapters/active_record/model.rb,
lib/flipper/adapters/active_record/feature.rb

Defined Under Namespace

Classes: Feature, Gate, Model

Constant Summary collapse

VALUE_TO_TEXT_WARNING =
<<-EOS
  Your database needs to be migrated to use the latest Flipper features.
  Run `rails generate flipper:update` and `rails db:migrate`.
EOS

Instance Method Summary collapse

Constructor Details

#initialize(options = {}) ⇒ ActiveRecord

Public: Initialize a new ActiveRecord adapter instance.

name - The Symbol name for this adapter. Optional (default :active_record) feature_class - The AR class responsible for the features table. gate_class - The AR class responsible for the gates table.

Allowing the overriding of name is so you can differentiate multiple instances of this adapter from each other, if, for some reason, that is a thing you do.

Allowing the overriding of the default feature/gate classes means you can roll your own tables and what not, if you so desire.



31
32
33
34
35
# File 'lib/flipper/adapters/active_record.rb', line 31

def initialize(options = {})
  @name = options.fetch(:name, :active_record)
  @feature_class = options.fetch(:feature_class) { Flipper::Adapters::ActiveRecord::Feature }
  @gate_class = options.fetch(:gate_class) { Flipper::Adapters::ActiveRecord::Gate }
end

Instance Method Details

#add(feature) ⇒ Object

Public: Adds a feature to the set of known features.



43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
# File 'lib/flipper/adapters/active_record.rb', line 43

def add(feature)
  with_write_connection(@feature_class) do
    @feature_class.transaction(requires_new: true) do
      begin
        # race condition, but add is only used by enable/disable which happen
        # super rarely, so it shouldn't matter in practice
        unless @feature_class.where(key: feature.key).exists?
          @feature_class.create!(key: feature.key)
        end
      rescue ::ActiveRecord::RecordNotUnique
        # already added
      end
    end
  end

  true
end

#clear(feature) ⇒ Object

Public: Clears the gate values for a feature.



73
74
75
76
# File 'lib/flipper/adapters/active_record.rb', line 73

def clear(feature)
  with_write_connection(@gate_class) { @gate_class.where(feature_key: feature.key).destroy_all }
  true
end

#disable(feature, gate, thing) ⇒ Object

Public: Disables a gate for a given thing.

feature - The Flipper::Feature for the gate. gate - The Flipper::Gate to disable. thing - The Flipper::Type being disabled for the gate.

Returns true.



162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
# File 'lib/flipper/adapters/active_record.rb', line 162

def disable(feature, gate, thing)
  case gate.data_type
  when :boolean
    clear(feature)
  when :integer
    set(feature, gate, thing)
  when :json
    with_write_connection(@gate_class) do
      delete(feature, gate)
    end
  when :set
    with_write_connection(@gate_class) do
      @gate_class.where(feature_key: feature.key, key: gate.key, value: thing.value).destroy_all
    end
  else
    unsupported_data_type gate.data_type
  end

  true
end

#enable(feature, gate, thing) ⇒ Object

Public: Enables a gate for a given thing.

feature - The Flipper::Feature for the gate. gate - The Flipper::Gate to enable. thing - The Flipper::Type being enabled for the gate.

Returns true.



138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
# File 'lib/flipper/adapters/active_record.rb', line 138

def enable(feature, gate, thing)
  case gate.data_type
  when :boolean
    set(feature, gate, thing, clear: true)
  when :integer
    set(feature, gate, thing)
  when :json
    set(feature, gate, thing, json: true)
  when :set
    enable_multi(feature, gate, thing)
  else
    unsupported_data_type gate.data_type
  end

  true
end

#featuresObject

Public: The set of known features.



38
39
40
# File 'lib/flipper/adapters/active_record.rb', line 38

def features
  with_connection(@feature_class) { @feature_class.distinct.pluck(:key).to_set }
end

#get(feature) ⇒ Object

Public: Gets the values for all gates for a given feature.

Returns a Hash of Flipper::Gate#key => value.



81
82
83
84
# File 'lib/flipper/adapters/active_record.rb', line 81

def get(feature)
  gates = with_connection(@gate_class) { @gate_class.where(feature_key: feature.key).pluck(:key, :value) }
  result_for_gates(feature, gates)
end

#get_all(**kwargs) ⇒ Object



103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
# File 'lib/flipper/adapters/active_record.rb', line 103

def get_all(**kwargs)
  with_connection(@feature_class) do |connection|
    # query the gates from the db in a single query
    features = ::Arel::Table.new(@feature_class.table_name.to_sym)
    gates = ::Arel::Table.new(@gate_class.table_name.to_sym)
    rows_query = features.join(gates, ::Arel::Nodes::OuterJoin)
      .on(features[:key].eq(gates[:feature_key]))
      .project(features[:key].as('feature_key'), gates[:key], gates[:value])
    gates = connection.select_rows(rows_query)

    # group the gates by feature key
    grouped_gates = gates.inject({}) do |hash, (feature_key, key, value)|
      hash[feature_key] ||= []
      hash[feature_key] << [key, value]
      hash
    end

    # build up the result hash
    result = Hash.new { |hash, key| hash[key] = default_config }
    features = grouped_gates.keys.map { |key| Flipper::Feature.new(key, self) }
    features.each do |feature|
      result[feature.key] = result_for_gates(feature, grouped_gates[feature.key])
    end
    result.default_proc = nil
    result
  end
end

#get_multi(features) ⇒ Object



86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
# File 'lib/flipper/adapters/active_record.rb', line 86

def get_multi(features)
  with_connection(@gate_class) do
    gates = @gate_class.where(feature_key: features.map(&:key)).pluck(:feature_key, :key, :value)
    grouped_gates = gates.inject({}) do |hash, (feature_key, key, value)|
      hash[feature_key] ||= []
      hash[feature_key] << [key, value]
      hash
    end

    result = {}
    features.each do |feature|
      result[feature.key] = result_for_gates(feature, grouped_gates[feature.key])
    end
    result
  end
end

#remove(feature) ⇒ Object

Public: Removes a feature from the set of known features.



62
63
64
65
66
67
68
69
70
# File 'lib/flipper/adapters/active_record.rb', line 62

def remove(feature)
  with_write_connection(@feature_class) do
    @feature_class.transaction do
      @feature_class.where(key: feature.key).destroy_all
      clear(feature)
    end
  end
  true
end

#unsupported_data_type(data_type) ⇒ Object

Private



184
185
186
# File 'lib/flipper/adapters/active_record.rb', line 184

def unsupported_data_type(data_type)
  raise "#{data_type} is not supported by this adapter"
end